/***************************************************************************
 *   Copyright (C) 2005-2007 by Patrick Steiner                            *
 *   patrick.steiner@a1.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QDebug>

extern "C" {
#include <gnokii.h>
}

#include "device.h"

static const char *manufacturer, *model, *revision, *imei;
static struct gn_statemachine *state;
gn_data *data;

static const char *GNOKIIRC_TEMP = "[global]\nport = %s\nmodel = %s\nconnection = %s\nrfcomm_channel = %d\n\
[logging]\ndebug = off\nrlpdebug = off\nxdebug = off\n";

char *GnokiiRcFile = NULL;

// thanks goes to: https://garage.maemo.org/plugins/scmsvn/viewcvs.php/gnokii_utils.c?root=selfone&rev=67&view=markup
static const char *creategnokiirc(const char *phoneport, const char *phonemodel, const char *connectiontype, const int *rf_channel)
{
	char *temp = strdup("/tmp/past_gnokii_XXXXXX");
	FILE *file;

	int fd;
	fd = mkstemp(temp);

	if(fd <= -1) {
		qWarning("ERROR: Cant create temp config file");
		return NULL;
	}

	GnokiiRcFile = temp;
	file = fopen(GnokiiRcFile, "w+");
	fprintf (file, GNOKIIRC_TEMP, phoneport, phonemodel, connectiontype, rf_channel);
	fclose (file);

	return GnokiiRcFile;
}

void
deletegnokiirc ()
{
	if(!GnokiiRcFile)
        	return;

	unlink(GnokiiRcFile);
	free(GnokiiRcFile);
	GnokiiRcFile = NULL;
}

static QString businit(const char *phoneport, const char *phonemodel, const char *connectiontype, const int *rf_channel)
{
	gn_error error;

	const char *file = creategnokiirc(phoneport, phonemodel, connectiontype, rf_channel);

	// for testing only
	error = gn_lib_phoneprofile_load_from_file(file, NULL, &state);
	//error = gn_lib_phoneprofile_load(NULL, &state);
	if (GN_ERR_NONE == error) {
		error = gn_lib_phone_open(state);
	}

	if (GN_ERR_NONE != error) {
		qWarning() << "ERROR:" << gn_error_print(error);
		return "ERROR: While connection to the phone";
	}

	data = &state->sm_data;

        // identify phone
	manufacturer = gn_lib_get_phone_manufacturer(state);
	model = gn_lib_get_phone_model(state);
	revision = gn_lib_get_phone_revision(state);
	imei = gn_lib_get_phone_imei(state);

	qDebug() << "dbg() manufacturer:" << manufacturer;
	qDebug() << "dbg() model:" << model;
	qDebug() << "dbg() revision:" << revision;
	qDebug() << "dbg() imei:" << imei;

	return QString::null;
}

static void busterminate()
{
	gn_lib_phone_close(state);
	gn_lib_phoneprofile_free(&state);
	gn_lib_library_free();

	deletegnokiirc();
}

static QString sendsms(const char *phonenumber, const char *message)
{
	gn_sms sms;
	gn_error error;

	qDebug() << "dbg(): Phonenumber:" << phonenumber;
	qDebug() << "dbg(): Message:" << message;

	/* The memory is zeroed here */
	gn_sms_default_submit(&sms);

	/* Set the destination number */
	memset(&sms.remote.number, 0, sizeof(sms.remote.number));
	snprintf(sms.remote.number, sizeof(sms.remote.number), "%s", phonenumber);

	if(sms.remote.number[0] == '+')
		sms.remote.type = GN_GSM_NUMBER_International;
	else
		sms.remote.type = GN_GSM_NUMBER_Unknown;

	/* Set the smsc number */
	data->message_center = (gn_sms_message_center *) calloc(1, sizeof(gn_sms_message_center));
	data->message_center->id = 1;

	if(gn_sm_functions(GN_OP_GetSMSCenter, data, state) == GN_ERR_NONE) {
		snprintf(sms.smsc.number, sizeof(sms.smsc.number), "%s", data->message_center->smsc.number);
		sms.smsc.type = data->message_center->smsc.type;
	}

	/* Set the message data */
	sms.user_data[0].type = GN_SMS_DATA_Text;
	snprintf((char *)sms.user_data[0].u.text, sizeof(sms.user_data[0].u.text), "%s", message);
	sms.user_data[0].length = strlen (message);

	sms.user_data[1].type = GN_SMS_DATA_None;

	data->sms = &sms;

	/* Send the message. */
	error = gn_sms_send(data, state);

	if(error == GN_ERR_NONE) {
		return QString::null;
	} else {
		qWarning() << "ERROR: Message send failed" << gn_error_print(error);
		return "ERROR: While sending message";
	}
}

Device::Device()
{
	initOK = false;
}

Device::~Device()
{
	if(initOK) {
		//qDebug() << "dbg() closing" << __FUNCTION__;
		busterminate();
	}
}

QString Device::initDevice(const char *phoneport, const char *phonemodel, const char *connectiontype, const int *rf_channel)
{
	QString errStr = businit(phoneport, phonemodel, connectiontype, rf_channel);

	if (!errStr.isEmpty()) {
		initOK = false;
		return errStr;
	}

	initOK = true;
	return QString::null;
}

QString Device::sendSMS(const char *phonenumber, const char *message)
{
	QString errStr = sendsms(phonenumber, message);

	if(!errStr.isEmpty()) {
		return errStr;
	}

	return QString::null;
}
