/***************************************************************************
 *   Copyright (C) 2005-2007 by Patrick Steiner                            *
 *   patrick.steiner@a1.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>

#include "app.h"
#include "config.h"
#include "smssendwindow.h"
#include "device.h"
#include "settingsdlg.h"
#include "phonebook.h"
#include "smslog.h"

SmsSendWindow::SmsSendWindow(QWidget *parent): QMainWindow(parent)
{
	setupUi(this);

	phonechars = 0;
	smschars = 0;

	setSendButton();

	readSettings();

	createActions();
	setupTrayIcon();
	setupStatusbar();

	connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
		this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	connect(btnSend, SIGNAL(clicked(bool)), this, SLOT(sendSMS()));
	connect(btnCancel, SIGNAL(clicked(bool)), this, SLOT(cancelSMS()));

	trayIcon->show();

	// setup contact list
	PhoneBook *pBook = new PhoneBook(this);
	QStringList cList = pBook->getAllContacts();

	QCompleter *completer = new QCompleter(cList, this);
	completer->setCaseSensitivity(Qt::CaseInsensitive);
	lnPhonenumber->setCompleter(completer);
}

void SmsSendWindow::setVisible(bool visible)
{
	 if(!visible) {
		trayIconMenu->addAction(restoreAction);
		trayIconMenu->removeAction(minimizeAction);
	} else {
		trayIconMenu->removeAction(restoreAction);
		trayIconMenu->addAction(minimizeAction);
	}
	trayIconMenu->addSeparator();

	trayIconMenu->addAction(settingsDlgAction);
	trayIconMenu->addAction(aboutAction);
	trayIconMenu->addSeparator();
	trayIconMenu->addAction(quitAction);

	trayIcon->setContextMenu(trayIconMenu);
	
	QWidget::setVisible(visible);
}

void SmsSendWindow::readSettings()
{
	QSettings settings("past");

	multisms = settings.value("multisms").toBool();

	storeSMS = settings.value("storesms").toBool();

	phonePort = settings.value("phonePort").toString();
	phoneModel = settings.value("phoneModel").toString();
	phoneConnectionType = settings.value("phoneConnectionType").toString();
	phoneRfChannel = settings.value("phoneRfChannel").toInt();
}

void SmsSendWindow::setupStatusbar()
{
	stbCharCount = new QLabel();
	statusBar()->addPermanentWidget(stbCharCount);
	connect(editMessage, SIGNAL(textChanged()), this, SLOT(updateCharStats()));
	connect(lnPhonenumber, SIGNAL(textChanged(QString)), this, SLOT(phonenumberChanged()));
	updateCharStats();
}

void SmsSendWindow::updateCharStats()
{
	QString text, output;
	int messages, charsLeft;

	text = editMessage->toPlainText().toLatin1();
	smschars = text.length();

	if(multisms) {
		if(smschars > 0)
			messages = (smschars / SMS_MESSAGE_LENGTH) + 1;
		else
			messages = 0;

		charsLeft = (SMS_MESSAGE_LENGTH * messages) - smschars;

		output = tr("Messages: %1, Characters left: %2").arg(messages).arg(charsLeft);

	} else {
		charsLeft = SMS_MESSAGE_LENGTH - smschars;

		if(charsLeft <= 0) {
			output = tr("Message too long");
		} else {
			output = tr("%1 Character(s) left").arg(charsLeft);
		}
	}

	setSendButton();

	stbCharCount->setText(output);
}

void SmsSendWindow::phonenumberChanged()
{
	QString phonenumber;

	phonenumber = lnPhonenumber->text().toLatin1();
	phonechars = phonenumber.length();

	setSendButton();
}

void SmsSendWindow::setSendButton()
{
	if(phonechars <= 0) {
		btnSend->setEnabled(false);
		return;
	}

	if(smschars <= 0) {
		btnSend->setEnabled(false);
		return;
	}

	if(!multisms && smschars >= SMS_MESSAGE_LENGTH) {
		btnSend->setEnabled(false);
	} else {
		btnSend->setEnabled(true);
	}
}

void SmsSendWindow::sendSMS()
{
	Device device;
	QByteArray tmpMessage, tmpPhoneNumber;
	PhoneBook *pBook = new PhoneBook(this);
	QString inputPhoneLabel;
	bool ok;

	inputPhoneLabel = lnPhonenumber->text();

	// check if phonenumber is an integer
	int y = inputPhoneLabel.toInt(&ok);

	//TODO: build a own better phonenumber check function
	// 1. convert name into numer (if it comes from the addressbook
	// 2. check if this number could be valid

	if(pBook->getPhoneNumberByName(inputPhoneLabel) != NULL) {
		tmpPhoneNumber = pBook->getPhoneNumberByName(inputPhoneLabel).toLatin1();
	} else if(inputPhoneLabel.startsWith("+")) {
		tmpPhoneNumber = inputPhoneLabel.toLatin1();
	} else if(ok == TRUE) {
		tmpPhoneNumber = inputPhoneLabel.toLatin1();
	} else {
		QMessageBox::critical(this,
					tr("%1 - Error").arg(APP_NAME),
					tr("Unable to send Message to %1").arg(inputPhoneLabel));
		return;
	}

	const char *pNumber = tmpPhoneNumber.data();

	tmpMessage = editMessage->toPlainText().toLatin1();
	const char *message = tmpMessage.data();

	if(phonePort.isEmpty()) {
		QMessageBox::critical(this, tr("%1 - Error").arg(APP_NAME), tr("No Phone Port"));
		return;
	}

	if(phoneModel.isEmpty()) {
		QMessageBox::critical(this, tr("%1 - Error").arg(APP_NAME), tr("No Phone Model"));
		return;
	}

	if(phoneConnectionType.isEmpty()) {
		QMessageBox::critical(this, tr("%1 - Error").arg(APP_NAME), tr("No Phone Connection Type"));
		return;
	}

	statusBar()->showMessage(tr("Connecting..."));
	qDebug("Connecting...");

	QString initErr = device.initDevice(phonePort.toLatin1(), phoneModel.toLatin1(), phoneConnectionType.toLatin1(), (const int*)phoneRfChannel);

	if(!initErr.isEmpty()) {
		qWarning() << initErr;
		statusBar()->showMessage(initErr, 5000);
		QMessageBox::critical(this, tr("%1 - Error").arg(APP_NAME), initErr);
		return;
	}

	statusBar()->showMessage(tr("Connected"));
	qDebug("Connected");

	statusBar()->showMessage(tr("Sending..."));
	qDebug("Sending...");

	QString sendErr = device.sendSMS(pNumber, message);

	if(!sendErr.isEmpty()) {
		qWarning() << sendErr;
		statusBar()->showMessage(sendErr, 5000);
		QMessageBox::critical(this, tr("%1 - Error").arg(APP_NAME), sendErr);
		return;
	}

	statusBar()->showMessage(tr("Send"), 5000);

	if(storeSMS) {
		SmsLogger *logger = new SmsLogger();

		//TODO: dont use inputPhoneLabel.text()
		//      should look like +12344654646 (John Doe)
		logger->logSMS(editMessage->toPlainText(), inputPhoneLabel);
	}

	clearSMSText();
}

void SmsSendWindow::cancelSMS()
{
	hide();
	clearSMSText();
}

void SmsSendWindow::clearSMSText()
{
	lnPhonenumber->clear();
	editMessage->clear();
}

void SmsSendWindow::closeEvent(QCloseEvent *event)
{
	if(trayIcon->isVisible()) {

		//TODO: add an information

		/*
		QPushButton *dontShowAgainBtn = new QPushButton(tr("Dont show this message again"));
		QMessageBox messageBox(QMessageBox::Information,
				       tr("%1 - Systray").arg(APP_NAME),
				        tr("The program will keep running in the "
					   "system tray. To terminate the program, "
					   "choose <b>Quit</b> in the context menu "
					   "of the system tray entry."),
				       QMessageBox::Ok | QMessageBox::Cancel, this);
		messageBox.addButton(dontShowAgainBtn, QMessageBox::AcceptRole);

		if(messageBox.clickedButton() == dontShowAgainBtn) {
			qDebug("dont show again");
		}

		switch(messageBox.exec()) {
			case QMessageBox::Cancel:
				qDebug("cancel");
				break;
			case QMessageBox::Ok:
			default:
				qDebug("ok | default");
				hide();
		}

		event->ignore();
		/*

		/*
		QMessageBox::information(this, tr("%1 - Systray").arg(APP_NAME),
				tr("The program will keep running in the "
				"system tray. To terminate the program, "
				"choose <b>Quit</b> in the context menu "
				"of the system tray entry."), QMessageBox::Ok | QMessageBox::Cancel);
		hide();
		event->ignore();
		*/

		hide();
		event->ignore();
	}
}

void SmsSendWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
	switch (reason) {
		case QSystemTrayIcon::Trigger:
		case QSystemTrayIcon::DoubleClick:
		case QSystemTrayIcon::MiddleClick:
			if(isVisible()) {
				hide();
			} else {
				showNormal();
			}
			break;
		default:
			;
	}
}

void SmsSendWindow::createActions()
{
	minimizeAction = new QAction(tr("Mi&nimize"), this);
	connect(minimizeAction, SIGNAL(triggered()), this, SLOT(hide()));

	restoreAction = new QAction(tr("&Restore"), this);
	connect(restoreAction, SIGNAL(triggered()), this, SLOT(showNormal()));

	aboutAction = new QAction(tr("&About"), this);
	connect(aboutAction, SIGNAL(triggered()), this, SLOT(showAboutDlg()));

	settingsDlgAction = new QAction(tr("&Configure"), this);
	connect(settingsDlgAction, SIGNAL(triggered()), this, SLOT(showSettingsDlg()));

	quitAction = new QAction(tr("&Quit"), this);
	connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
}

void SmsSendWindow::setupTrayIcon()
{
	trayIcon = new QSystemTrayIcon(this);
	trayIcon->setIcon(*Icon_AppIcon);

	trayIconMenu = new QMenu(this);
}

void SmsSendWindow::showAboutDlg()
{
	QMessageBox::about(this, tr("About %1").arg(APP_NAME),
				 tr("<b>%1</b><br>"
				    "Version: %2.%3.%4<br>"
				    "Author: Patrick Steiner").arg(APP_NAME)
							      .arg(VERSION_MAJOR)
							      .arg(VERSION_MINOR)
							      .arg(VERSION_PATCH));
}

void SmsSendWindow::showSettingsDlg()
{
	SettingsDialog *settingsdlg = new SettingsDialog(this);
}
